@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2016, 2020, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "socket_detail.h"
@@ -561,10 +561,6 @@ addrinfo* addrinfo_from_string(const char* host_name, unsigned short port)
   addrinfo* result = NULL;
   addrinfo hints = {};
   in6_addr addr = {};
-  char str_port[6];
-
-  if (std::sprintf(str_port, "%hu", port) < 0)
-    throw_error("Invalid port.");
 
   hints.ai_flags = AI_NUMERICSERV;
   hints.ai_family = AF_INET;
@@ -584,7 +580,11 @@ addrinfo* addrinfo_from_string(const char* host_name, unsigned short port)
     }
   }
 
-  int rc = getaddrinfo(host_name, str_port, &hints, &result);
+  // cast to unsigned because, unsigned short would use
+  // std::to_string(int)
+  int rc = getaddrinfo(host_name,
+                       std::to_string(static_cast<unsigned>(port)).c_str(),
+                       &hints, &result);
 
 #ifdef EAI_SYSTEM
   if (EAI_SYSTEM == rc && errno)
@@ -643,7 +643,7 @@ Socket connect(const char *host_name, unsigned short port,
   struct AddrInfoGuard
   {
     addrinfo* list;
-    ~AddrInfoGuard() { freeaddrinfo(list); }
+    ~AddrInfoGuard() NOEXCEPT { freeaddrinfo(list); }
   }
   guard = { host_list };
 
@@ -916,6 +916,13 @@ void recv(Socket socket, byte *buffer, size_t buffer_size)
 
   size_t bytes_received = 0;
 
+  /*
+    Note: In presence of timeouts recv_some() can return 0 which would lead
+    to an infinite loop here! See also bug#37278716.
+
+    A solution would be to throw error if timeout was hit?
+  */
+
   while (bytes_received != buffer_size)
     bytes_received += recv_some(socket, buffer + bytes_received, buffer_size - bytes_received, true);
 }
