@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <mysql/cdk.h>
@@ -193,9 +193,9 @@ void mysqlx::common::Settings_impl::Data::init_connection_attr()
   m_connection_attr["_platform"] = platform;
   m_connection_attr["_source_host"] =
       cdk::foundation::connection::get_local_hostname();
-  m_connection_attr["_client_name"] = CONCPP_NAME;
-  m_connection_attr["_client_version"] = CONCPP_VERSION;
-  m_connection_attr["_client_license"] = CONCPP_LICENSE;
+  m_connection_attr["_client_name"] = MYSQL_CONCPP_NAME;
+  m_connection_attr["_client_version"] = MYSQL_CONCPP_VERSION;
+  m_connection_attr["_client_license"] = MYSQL_CONCPP_LICENSE;
 }
 
 void mysqlx::common::Settings_impl::Data::clear_connection_attr()
@@ -323,8 +323,7 @@ void prepare_options(
   // Set TLS options
 
   /*
-    By default ssl-mode is REQUIRED. If ssl-mode was not explicitly set but
-    ssl-ca was, then mode defaults to VERIFY_CA.
+    By default ssl-mode is REQUIRED.
   */
 
   unsigned mode = unsigned(SSL_mode::REQUIRED);
@@ -335,25 +334,18 @@ void prepare_options(
     mode_set = true;
     mode = (unsigned)settings.get(Option::SSL_MODE).get_uint();
   }
-  else if (settings.has_option(Option::SSL_CA))
-  {
-    mode_set = true;
-    mode = unsigned(SSL_mode::VERIFY_CA);
-  }
 
   if (socket && mode_set && mode >= unsigned(SSL_mode::REQUIRED))
   {
     throw_error("SSL connection over Unix domain socket requested.");
   }
 
+#ifdef WITH_SSL
   if (unsigned(SSL_mode::DISABLED) == mode)
   {
-#ifdef WITH_SSL
     opts.set_tls(TLS_options::SSL_MODE::DISABLED);
-#endif
   }
   else
-#ifdef WITH_SSL
   {
     socket = true;  // so that PLAIN auth method is used below
 
@@ -364,7 +356,12 @@ void prepare_options(
       switch (opt_val.first)
       {
       case Option::TLS_VERSIONS:
+        try {
         tls_opt.add_version(opt_val.second.get_string());
+      }  catch (const cdk::Error&) {
+          //unexpected TLS versions are not errors. Only if no valid tls version
+          //is available, an error will be thrown below.
+        }
         break;
       case Option::TLS_CIPHERSUITES:
         tls_opt.add_ciphersuite(opt_val.second.get_string());
@@ -393,6 +390,13 @@ void prepare_options(
 
     if (settings.has_option(Option::SSL_CA))
       tls_opt.set_ca(settings.get(Option::SSL_CA).get_string());
+    if(settings.has_option(Option::SSL_CAPATH))
+      tls_opt.set_ca_path(settings.get(Option::SSL_CAPATH).get_string());
+    if (settings.has_option(Option::SSL_CRL))
+      tls_opt.set_crl(settings.get(Option::SSL_CRL).get_string());
+    if(settings.has_option(Option::SSL_CRLPATH))
+      tls_opt.set_crl_path(settings.get(Option::SSL_CRLPATH).get_string());
+
     opts.set_tls(tls_opt);
   }
 #endif
@@ -501,9 +505,6 @@ void Settings_impl::get_data_source(cdk::ds::Multi_source &src)
     If DNS+SRV is not used, get list of hosts from the settings.
   */
 
-  // if priorities were not set explicitly, assign decreasing starting from 100
-  int prio = m_data.m_user_priorities ? -1 : 100;
-
   /*
     Look for a priority after host/socket setting. If explicit priorities
     are used, then we expect the priority setting to be present and we throw
@@ -599,7 +600,7 @@ void Settings_impl::get_data_source(cdk::ds::Multi_source &src)
     throw_error("Unix socket connections not supported on Windows platform.");
   };
 #else
-  auto add_socket = [this, &src, &opts, check_prio](iterator &it, int prio) {
+  auto add_socket = [&src, &opts, check_prio](iterator &it, int prio) {
 
     assert(Session_option_impl::SOCKET == it->first);
 
@@ -707,7 +708,10 @@ Pooled_session::Pooled_session(cdk::ds::Multi_source &ds)
 
 Pooled_session::~Pooled_session()
 {
-  release();
+  //Let's catch any errors when releasing the session
+  try {
+    release();
+  } catch (...) {}
 }
 
 void Pooled_session::release()
@@ -802,33 +806,50 @@ try {
 catch (...)
 {}
 
-
-void Session_pool::close()
-{
+void Session_pool::close() {
   lock_guard guard(m_pool_mutex);
-  //First, close all sessions
-  for(auto &el : m_pool)
-  {
-    try{
-      // If there is a cleanup handler, call it before closing a session.
-      if (el.second.m_cleanup)
-        el.second.m_cleanup->cleanup();
-      el.first->close();
-    } catch(...)
-    {}
+  // First, close all sessions
+  bool first_iteration = true;
+  while (!m_pool.empty()) {
+    auto next_it = m_pool.begin();
+    for (auto it = next_it; it != m_pool.end(); it = next_it) {
+      ++next_it;
+      auto &el = *it;
+      if (el.second.m_cleanup) {  // It is use, needs to be cleanup before close
+        auto lock = el.second.m_cleanup->try_lock();
+        // On first iteration, will just try to lock, but on second iteration, it
+        // will wait untill lock is free
+        if (!first_iteration && !lock.owns_lock()) {
+        lock.lock();
+        }
+        if (lock.owns_lock()) {
+          try {
+            // call cleanup before closing a session.
+            el.second.m_cleanup->cleanup();
+            el.first->close();
+          } catch (...) {}
+          m_pool.erase(it);
+        }
+      } else {
+        try {
+          // unused sessions don't have the cleanup so, just close them.
+          el.first->close();
+        } catch (...) {}
+        m_pool.erase(it);
+      }
+    }
+    first_iteration = false;
   }
-  m_pool.clear();
 
-  //prevent changing m_pool_closed before getting release condition signal
+  // prevent changing m_pool_closed before getting release condition signal
   std::unique_lock<std::mutex> lock(m_reelase_mutex);
   m_pool_closed = true;
 
-  //Will notify all because, since pool is now closed, waiting pooled sessions
-  //will throw error!
+  // Will notify all because, since pool is now closed, waiting pooled sessions
+  // will throw error!
   m_release_cond.notify_all();
 }
 
-
 void Session_pool::release_session(cdk::shared_ptr<cdk::Session> &sess)
 {
   // Pool closed... nothing to do here!
@@ -952,8 +973,6 @@ Session_pool::get_session(Session_cleanup *cleanup)
 {
   lock_guard guard(m_pool_mutex);
 
-  bool use_blocklist = true;
-
   if (!m_pool_enable)
   {
     return std::shared_ptr<cdk::Session>(new cdk::Session(m_ds));
@@ -979,12 +998,29 @@ Session_pool::get_session(Session_cleanup *cleanup)
     avoiding the block-listed endpoints.
   */
 
+  /*
+    Keep the original block list so that, if non blocked servers fail to get a
+    session, previously blocked ones are tried.
+  */
+  Block_list original_block(m_block_list);
+
   if (m_pool.size() < m_max)
   {
     try
     {
-      auto block_list_filter = [this](size_t id) {
-        return m_block_list.is_block_listed(id);
+      auto block_list_filter = [this](size_t id, cdk::option_t opt) -> bool {
+        switch(opt.state())
+        {
+        case cdk::option_t::UNKNOWN:
+          return m_block_list.is_block_listed(id);
+        case cdk::option_t::YES:
+          m_block_list.remove(id);
+          break;
+        case cdk::option_t::NO:
+          m_block_list.add(id);
+          break;
+        }
+        return true;
       };
 
       auto ret = m_pool.emplace(
@@ -1013,14 +1049,32 @@ Session_pool::get_session(Session_cleanup *cleanup)
 
   /*
     If a session is still not found, and there is space in the pool,
-    try creating a new session this time ignoring the block-list.
+    try creating a new session this time using the block-listed ones.
   */
 
   if (m_pool.size() < m_max)
   {
+
+
+    auto block_list_filter = [this, &original_block](size_t id, cdk::option_t opt) -> bool {
+      switch(opt.state())
+      {
+      case cdk::option_t::UNKNOWN:
+        //Filter out sessions that were not block-listed
+        return !original_block.is_block_listed(id);
+      case cdk::option_t::YES:
+        m_block_list.remove(id);
+        break;
+      case cdk::option_t::NO:
+        m_block_list.add(id);
+        break;
+      }
+      return true;
+    };
+
     // If an exception is thrown, don't intercept it, let it propagate
     auto ret = m_pool.emplace(
-      cdk::shared_ptr<cdk::Session>(new cdk::Session(m_ds)),
+      cdk::shared_ptr<cdk::Session>(new cdk::Session(m_ds, block_list_filter)),
       Sess_data{ time_point::max(), cleanup }
     );
     return ret.first->first;
@@ -1039,7 +1093,7 @@ void Session_pool::time_to_live_cleanup()
   auto it = m_pool.begin();
   for(; it != m_pool.end(); )
   {
-    if (it->first.unique() && it->second.m_deadline < current_time)
+    if (it->first.use_count()==1 && it->second.m_deadline < current_time)
     {
       // Note: removed session is not active and does not need calling
       // of the cleanup handler.
