@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2017, 2020, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef MYSQLX_COMMON_SETTINGS_INT_H
@@ -768,17 +768,6 @@ Settings_impl::Setter::set_option<Settings_impl::Session_option_impl::SSL_MODE>(
     throw_error("secure connection requested but SSL is not supported")
 #endif
 
-  switch (m_data.m_ssl_mode)
-  {
-  case SSL_mode::VERIFY_CA:
-  case SSL_mode::VERIFY_IDENTITY:
-    break;
-
-  default:
-    if (m_data.m_ssl_ca)
-      throw_error("SSL_MODE ... not valid when SSL_CA is set");
-  }
-
   add_option(Session_option_impl::SSL_MODE, val);
 }
 
@@ -793,17 +782,6 @@ Settings_impl::Setter::set_option<Settings_impl::Session_option_impl::SSL_CA>(
   throw_error("SSL_CA option specified but SSL is not supported")
 #endif
 
-  switch (m_data.m_ssl_mode)
-  {
-  case SSL_mode::VERIFY_CA:
-  case SSL_mode::VERIFY_IDENTITY:
-  case SSL_mode::LAST:
-    break;
-
-  default:
-    throw_error("SSL_CA option is not compatible with SSL_MODE ...");
-  }
-
   m_data.m_ssl_ca = true;
   add_option(Session_option_impl::SSL_CA, val);
 }
@@ -1127,19 +1105,6 @@ void Settings_impl::Setter::add_option(int opt, const T &val)
       m_opt_set.insert(opt);  // needed for double check when m_multi is false
       return;
     }
-    // if multi mode not enabled, fall-through to check for doubled option
-
-  default:
-    // Check for doubled option
-    if (0 < m_opt_set.count(opt))
-    {
-      std::string msg = "Option ";
-      msg += option_name(opt);
-      msg += " defined twice";
-      throw_error(msg.c_str());
-      return;
-    }
-    m_opt_set.insert(opt);
   }
 
   auto it = options.begin();
